unit ESBMaths;

{ Contains useful Mathematical routines for Delphi 3

	 1998 ESB Consultancy

	These routines are used by ESB Consultancy within the
	development of their Customised Applications, and have been
	under Development since the early Turbo Pascal days.
	Many of the routines were developed for specific needs

	ESB Consultancy retains full copyright.

	ESB Consultancy grants users of this code royalty free rights
	to do with this code as they wish.

	ESB Consultancy makes no guarantees nor excepts any liabilities
	due to the use of these routines

	We does ask that if this code helps you in you development
	that you send as an email mailto:esb@gold.net.au or even
	a local postcard. It would also be nice if you gave us a
	mention in your About Box or Help File.

	ESB Consultancy Home Page: http://www.gold.net.au/~esb

	Mail Address: PO Box 2259, Boulder, WA 6432 AUSTRALIA

	v1.00 17 April 1998 - first public/freeware release

}

interface

//{$DEFINE UseMath} 	// comment out this line if you don't want to use the Delphi Math Library
  	// Desktop versions of Delphi do not include the Math Library

type
	TBitList = Word;
	{ Used for a Bit List of 16 bits from 15 -> 0 }

var
	ESBTolerance: Double = 0.00001;



function SameFloat (const X1, X2: Extended): Boolean;

{ Returns True if X1 and X2 are within ESBTolerance of each other }

function FloatIsZero (const X: Extended): Boolean;

{ Returns True if X is within ESBTolerance of 0 }

procedure IncLim (var B: Byte; const Limit: Byte);

 { Increments a Byte up to Limit. If B >= Limit no increment
  occurs. }

procedure IncLimSI (var B: ShortInt; const Limit: ShortInt);

 { Increments a ShortInt up to Limit. If B >= Limit no increment
  occurs. }

procedure IncLimW (var B: Word; const Limit: Word);

 { Increments a Word up to Limit. If B >= Limit no increment
  occurs. }

procedure IncLimI (var B: Integer; const Limit: Integer);

 { Increments an Integer up to Limit. If B >= Limit no increment
  occurs. }

procedure IncLimL (var B: LongInt; const Limit: LongInt);

 { Increments a LongInt up to Limit. If B >= Limit no increment
  occurs. }

procedure DecLim (var B: Byte; const Limit: Byte);

 { Decrements a Byte down to Limit. If B <= Limit no increment
  occurs. BASM }

procedure DecLimSI (var B: ShortInt; const Limit: ShortInt);

 { Decrements a ShortInt down to Limit. If B <= Limit no increment
  occurs. }

procedure DecLimW (var B: Word; const Limit: Word);

 { Decrements a Word down to Limit. If B <= Limit no increment
  occurs. }

procedure DecLimI (var B: Integer; const Limit: Integer);

 { Decrements an Integer down to Limit. If B <= Limit no increment
  occurs. }

procedure DecLimL (var B: LongInt; const Limit: LongInt);

 { Decrements a LongInt down to Limit. If B <= Limit no increment
  occurs. }

function MaxB (const B1, B2: Byte): Byte;

 { Returns the maximum value between two Bytes. BASM }

function MinB (const B1, B2: Byte): Byte;

 { Returns the minimum value between two Bytes. BASM }

function MaxSI (const B1, B2: ShortInt): ShortInt;

 { Returns the maximum value between two ShortInts. }

function MinSI (const B1, B2: ShortInt): ShortInt;

 { Returns the minimum value between two ShortInts. }

function MaxW (const B1, B2: Word): Word;

 { Returns the maximum value between two Words. BASM }

function MinW (const B1, B2: Word): Word;

 { Returns the minimum value between two Words. BASM }

function MaxI (const B1, B2: Integer): Integer;

 { Returns the maximum value between two Integers. }

function MinI (const B1, B2: Integer): Integer;

 { Returns the minimum value between two Integers. }

function MaxL (const B1, B2: LongInt): LongInt;

 { Returns the maximum value between two LongInts. }

function MinL (const B1, B2: LongInt): LongInt;

 { Returns the minimum value between two LongInts. }

procedure SwapB (var B1, B2: Byte); register;

 { Swap Two Bytes. BASM - using Registers }

procedure SwapSI (var B1, B2: ShortInt); register;

 { Swap Two ShortInts. BASM - using Registers }

procedure SwapW (var B1, B2: Word); register;

 { Swap Two Words. BASM - using Registers }

procedure SwapI (var B1, B2: SmallInt); register;

 { Swap Two Integers. BASM - using Registers }

procedure SwapL (var B1, B2: LongInt); register;

 { Swap Two LongInts. BASM - using Registers }

function Sign (const B: LongInt): ShortInt;

 { Returns  -1  if B < 0
     0  if B = 0
     1   if B > 0  BASM }

function Max4Word (const X1, X2, X3, X4: Word): Word;

 { Returns the Maximum of 4 Words - BASM }

function Min4Word (const X1, X2, X3, X4: Word): Word;

 { Returns the Minimum of 4 Words - BASM }

function Max3Word (const X1, X2, X3: Word): Word;

 { Returns the Maximum of 3 Words - BASM }

function Min3Word (const X1, X2, X3: Word): Word;

 { Returns the Minimum of 3 Words - BASM }

function MaxBArray (const B: array of Byte): Byte;

 { Returns the Maximum of an array of Bytes }

function MaxWArray (const B: array of Word): Word;

 { Returns the Maximum of an array of Words }

function MaxSIArray (const B: array of ShortInt): ShortInt;

 { Returns the Maximum of an array of ShortInts }

function MaxIArray (const B: array of Integer): Integer;

 { Returns the Maximum of an array of Integers }

function MaxLArray (const B: array of LongInt): LongInt;

 { Returns the Maximum of an array of LongInts }

function MinBArray (const B: array of Byte): Byte;

 { Returns the Minimum of an array of Bytes }

function MinWArray (const B: array of Word): Word;

 { Returns the Minimum of an array of Words }

function MinSIArray (const B: array of ShortInt): ShortInt;

 { Returns the Minimum of an array of ShortInts }

function MinIArray (const B: array of Integer): Integer;

 { Returns the Minimum of an array of Integers }

function MinLArray (const B: array of LongInt): LongInt;

 { Returns the Minimum of an array of LongInts }

function SumBArray (const B: array of Byte): Byte;

 { Returns the Sum of an array of Bytes.
  All Operation in Bytes }

function SumBArray2 (const B: array of Byte): Word;

 { Returns the Sum of an array of Bytes.
  All Operation in Words }

function SumSIArray (const B: array of ShortInt): ShortInt;

 { Returns the Sum of an array of ShortInts.
  All Operation in ShortInts }

function SumSIArray2 (const B: array of ShortInt): Integer;

 { Returns the Sum of an array of ShortInts.
  All Operation in Integers }

function SumWArray (const B: array of Word): Word;

 { Returns the Sum of an array of Words.
  All Operation in Words }

function SumWArray2 (const B: array of Word): LongInt;

 { Returns the Sum of an array of Words.
  All Operation in Longints }

function SumIArray (const B: array of Integer): Integer;

 { Returns the Sum of an array of Integers.
  All Operation in Integers }

function SumLArray (const B: array of LongInt): LongInt;

 { Returns the Sum of an array of Longints.
  All Operation in Longints }

function ISqrt (const I: SmallInt): SmallInt; register;

{ ISqrt (I) computes INT (SQRT (I)), that is, the integral part of the
  square root of integer I. It does not check for negative arguments.
  For all arguments 0..MaxInt the correct result is returned. The
  algorithm exploits the following property:
   n
   n**2 =  Sigma (2i-1)
   i=1

   Routine by Norbert Joffa }

function Get87ControlWord: TBitList;

 { Returns the 80x87 Control Word
  15-12 Reserved
     On 8087/80287 12 was Infinity Control
      0 Projective
	 1 Affine
  11-10 Rounding Control
    00 Round to nearest even
    01 Round Down
    10 Round Up
    11 Chop - Truncate towards Zero
  9-8  Precision Control
    00 24 bits Single Precision
    01 Reserved
    10 53 bits Double Precision
    11 64 bits Extended Precision (Default)
  7-6  Reserved
     On 8087 7 was Interrupt Enable Mask
  5  Precesion Exception Mask
  4  Underflow Exception Mask
  3  Overflow Exception Mask
  2  Zero Divide Exception Mask
  1  Denormalised Operand Exception Mask
  0  Invalid Operation Exception Mask

  BASM }

procedure Set87ControlWord (const CWord: TBitList); register;

 { Sets the 80x87 Control Word
  15-12 Reserved
	On 8087/80287 12 was Infinity Control
      0 Projective
      1 Affine
  11-10 Rounding Control
    00 Round to nearest even
    01 Round Down
    10 Round Up
    11 Chop - Truncate towards Zero
  9-8  Precision Control
    00 24 bits Single Precision
    01 Reserved
    10 53 bits Double Precision
    11 64 bits Extended Precision (Default)
  7-6  Reserved
     On 8087 7 was Interrupt Enable Mask
  5  Precesion Exception Mask
  4  Underflow Exception Mask
  3  Overflow Exception Mask
  2  Zero Divide Exception Mask
  1  Denormalised Operand Exception Mask
  0  Invalid Operation Exception Mask

  BASM }

{#Z+}
{*** Misc Mathematical Routines ***}
{#Z-}
function Sgn (const X: Extended): ShortInt;

 { Returns  -1 if X < 0
     0 if X = 0
     1 if X > 0
 }

function Distance (const X1, Y1, X2, Y2: Extended): Extended;

 { Returns the straight line Distance between (X1, Y1)
  and (X2, Y2) }

function ExtMod (const X, Y: Extended): Extended;

 { Performs Floating Point Modulus.
  ExtMod := X - Floor ( X / Y ) * Y
 }

function ExtRem (const X, Y: Extended): Extended;

 { Performs Floating Point Remainder.
  ExtRem := X - Int ( X / Y ) * Y
 }

function CompMOD (const X, Y: Comp): Comp;

 { Returns X mod Y for Comp Data Types }

{#Z+}
{*** Conversions ***}
{#Z-}
procedure Polar2XY (const Rho, Theta: Extended; var X, Y: Extended);

 { Converts Polar Co-ordinates into Cartesion Co-ordinates }

procedure XY2Polar (const X, Y: Extended; var Rho, Theta: Extended);

 { Converts Cartesian Co-ordinates to Polar Co-ordinates }

function DMS2Extended (const Degs, Mins, Secs: Extended): Extended;

 { Converts Degrees/Minutes/Seconds into an Extended Real }

procedure Extended2DMS (const X: Extended; var Degs, Mins, Secs: Extended);

 { Converts an Extended Real into Degrees/Minutes/Seconds }
function MaxExt (const X, Y: Extended): Extended;

 { Returns the Maximum of two Extended Reals }

function MinExt (const X, Y: Extended): Extended;

 { Returns the Minimum of two Extended Reals }

function MaxEArray (const B: array of Extended): Extended;

 { Returns the Maximum of an array of Extended Reals }

function MinEArray (const B: array of Extended): Extended;

 { Returns the Minimum of an array of Extended Reals }

function MaxSArray (const B: array of Single): Single;

 { Returns the Maximum of an array of Single Reals }

function MinSArray (const B: array of Single): Single;

 { Returns the Minimum of an array of Single Reals }

function MaxCArray (const B: array of Comp): Comp;

 { Returns the Maximum of an array of Comp Reals }

function MinCArray (const B: array of Comp): Comp;

 { Returns the Minimum of an array of Comp Reals }

function SumSArray (const B: array of Single): Single;

 { Returns the Sum of an Array of Single Reals }

function SumEArray (const B: array of Extended): Extended;

 { Returns the Sum of an Array of Extended Reals }

function SumCArray (const B: array of Comp): Comp;

 { Returns the Sum of an Array of Comp Reals }

implementation

{$IFDEF UseMath}
uses
	Math;
{$ENDIF}

procedure IncLim (var B: Byte; const Limit: Byte);
begin
	if B < Limit then
		Inc (B);
end;

procedure IncLimSI (var B: ShortInt; const Limit: ShortInt);
begin
	if B < Limit then
		Inc (B);
end;

procedure IncLimW (var B: Word; const Limit: Word);
begin
	if B < Limit then
		Inc (B);
end;

procedure IncLimI (var B: Integer; const Limit: Integer);
begin
	if B < Limit then
		Inc (B);
end;

procedure IncLimL (var B: LongInt; const Limit: LongInt);
begin
	if B < Limit then
		Inc (B);
end;

procedure DecLim (var B: Byte; const Limit: Byte);
begin
	if B > Limit then
		Dec (B);
end;

procedure DecLimSI (var B: ShortInt; const Limit: ShortInt);
begin
	if B > Limit then
		Dec (B);
end;

procedure DecLimW (var B: Word; const Limit: Word);
begin
	if B > Limit then
		Dec (B);
end;

procedure DecLimI (var B: Integer; const Limit: Integer);
begin
	if B > Limit then
		Dec (B);
end;

procedure DecLimL (var B: LongInt; const Limit: LongInt);
begin
	if B > Limit then
		Dec (B);
end;

function MaxB (const B1, B2: Byte): Byte;
begin
	if B1 > B2 then
		Result := B1
	 else
		Result := B2;
end;

function MinB (const B1, B2: Byte): Byte;
begin
	if B1 < B2 then
		Result := B1
	else
		Result := B2;
end;

function MaxSI (const B1, B2: ShortInt): ShortInt;
begin
	if B1 > B2 then
		Result := B1
	else
		Result := B2;
end;

function MinSI (const B1, B2: ShortInt): ShortInt;
begin
	if B1 < B2 then
		Result := B1
	else
		Result := B2;
end;

function MaxW (const B1, B2: Word): Word;
begin
	if B1 > B2 then
		Result := B1
	else
		Result := B2;
end;

function MinW (const B1, B2: Word): Word;
begin
	if B1 < B2 then
		Result := B1
	else
		Result := B2;
end;

function MaxI (const B1, B2: Integer): Integer;
begin
	if B1 > B2 then
		Result := B1
	else
		Result := B2;
end;

function MinI (const B1, B2: Integer): Integer;
begin
	if B1 < B2 then
		Result := B1
	else
		Result := B2;
end;

function MaxL (const B1, B2: LongInt): LongInt;
begin
	if B1 > B2 then
		Result := B1
	else
		Result := B2;
end;

function MinL (const B1, B2: LongInt): LongInt;
begin
	if B1 < B2 then
		Result := B1
	else
		Result := B2;
end;

procedure SwapB (var B1, B2: Byte); assembler;
asm
  mov  cl, Byte Ptr [EAX];
  mov  ch, Byte Ptr [EDX];
  mov  Byte Ptr [EDX], cl
  mov  Byte Ptr [EAX], ch
end;

procedure SwapSI (var B1, B2: ShortInt); assembler;
asm
  mov  cl, Byte Ptr [EAX];
  mov  ch, Byte Ptr [EDX];
  mov  Byte Ptr [EDX], cl
  mov  Byte Ptr [EAX], ch
end;

procedure SwapW (var B1, B2: Word); assembler;
asm
  push EBX
  mov  cx, Word Ptr [EAX];
  mov  bx, Word Ptr [EDX];
  mov  word Ptr [EDX], cx
  mov  word Ptr [EAX], bx
  pop EBX
end;

procedure SwapI (var B1, B2: SmallInt); assembler;
asm
  push EBX
  mov  cx, Word Ptr [EAX];
  mov  bx, Word Ptr [EDX];
  mov  word Ptr [EDX], cx
  mov  word Ptr [EAX], bx
  pop EBX
end;

procedure SwapL (var B1, B2: LongInt); assembler;
asm
  push EBX
  mov  ecx, [EAX];
  mov  ebx, [EDX];
  mov  [EDX], ecx
  mov  [EAX], ebx
  pop EBX
end;

function Sign (const B: LongInt): ShortInt;
begin
	if B < 0 then
		Result := -1
	else if B = 0 then
		Result := 0
	else
		Result := 1;
end;

function Max4Word (const X1, X2, X3, X4: Word): Word;
begin
	Result := X1;
	if X2 > Result then
		Result := X2;
	if X3 > Result then
		Result := X3;
	if X4 > Result then
		Result := X4;
end;

function Min4Word (const X1, X2, X3, X4: Word): Word;
begin
	Result := X1;
	if X2 < Result then
		Result := X2;
	if X3 < Result then
		Result := X3;
	if X4 < Result then
		Result := X4;
end;

function Max3Word (const X1, X2, X3: Word): Word; assembler;
begin
	Result := X1;
	if X2 > Result then
		Result := X2;
	if X3 > Result then
		Result := X3;
end;

function Min3Word (const X1, X2, X3: Word): Word; assembler;
begin
	Result := X1;
	if X2 < Result then
		Result := X2;
	if X3 < Result then
		Result := X3;
end;

function MaxBArray (const B: array of Byte): Byte;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I:= Low (B) + 1 to High (B) do
		if B [I] > Result then
			Result := B [I];
end;

function MaxWArray (const B: array of Word): Word;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I:= Low (B) + 1 to High (B) do
		if B [I] > Result then
		Result := B [I];
end;

function MaxIArray (const B: array of Integer): Integer;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I:= Low (B) + 1 to High (B) do
		if B [I] > Result then
			   Result := B [I];
end;

function MaxSIArray (const B: array of ShortInt): ShortInt;
var
	 I: Integer;
begin
	Result := B [Low (B)];
	for I := Low (B) + 1 to High (B) do
	if B [I] > Result then
		Result := B [I];
end;

function MaxLArray (const B: array of LongInt): LongInt;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I := Low (B) + 1 to High (B) do
		if B [I] > Result then
			Result := B [I];
end;

function MinBArray (const B: array of Byte): Byte;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I := Low (B) + 1 to High (B) do
		if B [I] < Result then
		   Result := B [I];
end;

function MinWArray (const B: array of Word): Word;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I := Low (B) + 1 to High (B) do
		if B [I] < Result then
			  Result := B [I];
end;

function MinIArray (const B: array of Integer): Integer;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I := Low (B) + 1 to High (B) do
		if B [I] < Result then
			 Result := B [I];
end;

function MinSIArray (const B: array of ShortInt): ShortInt;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I := Low (B) + 1 to High (B) do
		if B [I] < Result then
			Result := B [I];
end;

function MinLArray (const B: array of LongInt): LongInt;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I := Low (B) + 1 to High (B) do
		if B [I] < Result then
			Result := B [I];
end;

function ISqrt (const I: SmallInt): SmallInt; assembler;
asm
	push ebx

	MOV   CX, AX  { load argument }
	MOV   AX, -1  { init result }
	CWD           { init odd numbers to -1 }
	XOR   BX, BX  { init perfect squares to 0 }
@loop:
	INC   AX      { increment result }
	INC   DX      { compute }
	INC   DX      {  next odd number }
	ADD   BX, DX  { next perfect square }
	CMP   BX, CX  { perfect square > argument ? }
	JBE   @loop   { until square greater than argument }

	pop ebx
end;

function SumBArray (const B: array of Byte): Byte;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I:= Low (B) + 1 to High (B) do
		Result := Result + B [I];
end;

function SumBArray2 (const B: array of Byte): Word;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I:= Low (B) + 1 to High (B) do
		Result := Result + B [I];
end;

function SumSIArray (const B: array of ShortInt): ShortInt;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I:= Low (B) + 1 to High (B) do
		Result := Result + B [I];
end;

function SumSIArray2 (const B: array of ShortInt): Integer;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I:= Low (B) + 1 to High (B) do
		Result := Result + B [I];
end;

function SumWArray (const B: array of Word): Word;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I:= 0 to High (B) do
		Result := Result + B [I];
end;

function SumWArray2 (const B: array of Word): LongInt;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I:= Low (B) + 1 to High (B) do
		Result := Result + B [I];
end;

function SumIArray (const B: array of Integer): Integer;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I:= Low (B) + 1 to High (B) do
		Result := Result + B [I];
end;

function SumLArray (const B: array of LongInt): LongInt;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I:= Low (B) + 1 to High (B) do
		Result := Result + B [I];
end;

function Get87ControlWord: TBitList; assembler;
var
	Temp: Word;
asm
	fstcw [Temp]  		// Get '87 Control Word
	mov  ax, [Temp] 	// Leave in AX for function
end;

procedure Set87ControlWord (const CWord: TBitList); assembler;
var
	Temp: Word;
asm
	mov   [Temp], ax
	fldcw [Temp] 		// Load '87 Control Word
end;

procedure Polar2XY (const Rho, Theta: Extended; var X, Y: Extended);
begin
	{$IFDEF Math}
	SinCos (Theta, X, Y); // quite fast
	{$ELSE}
	X := Sin (Theta);
	Y := Cos (Theta);
	{$ENDIF}
	 X := Rho * X;
	 Y := Rho * Y;
end;

procedure XY2Polar (const X, Y: Extended; var Rho, Theta: Extended);
begin
	Rho := Sqrt (Sqr (X) + Sqr (Y));
	if Abs (X) > ESBTolerance then
		Theta := ArcTan (Y / X)
	else
		Theta := Sgn (Y) * Pi / 2.0
end;

function Sgn (const X: Extended): ShortInt;
begin
	if X < 0.0 then
		Result := -1
	else if X = 0.0 then
		Result := 0
	else
		Result := 1
end;

function DMS2Extended (const Degs, Mins, Secs: Extended): Extended;
begin
	Result := Degs + Mins / 60.0 + Secs / 3600.0
end;

procedure Extended2DMS (const X: Extended; var Degs, Mins, Secs: Extended);
var
	Y: Extended;
begin
	Degs := Int (X);
	Y := Frac (X) * 60;
	Mins := Int (Y);
	Secs := Frac (Y) * 60;
end;

function Distance (const X1, Y1, X2, Y2: Extended): Extended;
begin
	Result := Sqrt ( (X1 - X2) * (X1 - X2)  + (Y1 - Y2) * (Y1 - Y2))
end;

function ExtMod (const X, Y: Extended): Extended;
var
	Z: Extended;
begin
	 Result := X / Y;
	 Z := Int (Result);
	 if Result < 0 then
		Z := Z - 1.0;
	// Z now has Floor (X / Y)
	 Result := X - Z * Y
end;

function ExtRem (const X, Y: Extended): Extended;
begin
	Result := X - Int (X / Y) * Y
end;

function MaxExt (const X, Y: Extended): Extended;
begin
	if X > Y then
		Result := X
	else
		Result := Y
end;

function MinExt (const X, Y: Extended): Extended;
begin
	if X < Y then
		Result := X
	else
		Result := Y
end;

function CompMOD (const X, Y: Comp): Comp;
begin
	Result := X - Y * Int (X / Y)
end;

function MaxEArray (const B: array of Extended): Extended;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I := Low (B) + 1 to High (B) do
		if B [I] > Result then
			Result := B [I];
end;

function MinEArray (const B: array of Extended): Extended;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I := Low (B) + 1 to High (B) do
	if B [I] < Result then
		Result := B [I];
end;

function MaxSArray (const B: array of Single): Single;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I := Low (B) + 1 to High (B) do
		if B [I] > Result then
			Result := B [I];
end;

function MinSArray (const B: array of Single): Single;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I := Low (B) + 1 to High (B) do
		if B [I] < Result then
			  Result := B [I];
end;

function MaxCArray (const B: array of Comp): Comp;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I := Low (B) + 1 to High (B) do
		if B [I] > Result then
			Result := B [I];
end;

function MinCArray (const B: array of Comp): Comp;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I := Low (B) + 1 to High (B) do
		if B [I] < Result then
			Result := B [I];
end;

function SumSArray (const B: array of Single): Single;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I := Low (B) + 1 to High (B) do
		Result := Result + B [I];
end;

function SumEArray (const B: array of Extended): Extended;
var
	I: Integer;
begin
	Result := B [Low (B)];
	for I := Low (B) + 1 to High (B) do
		Result := Result + B [I];
end;

function SumCArray (const B: array of Comp): Comp;
var
	I: Integer;
begin
	Result := Low (B);
	for I := Low (B) + 1 to High (B) do
		Result := Result + B [I];
end;

function SameFloat (const X1, X2: Extended): Boolean;
begin
	Result := abs (X1 - X2) < ESBTolerance
end;

function FloatIsZero (const X: Extended): Boolean;
begin
	Result := abs (X) < ESBTolerance
end;

end.
